=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-10-20 05:28:45 +0000
+++ bzrlib/builtins.py	2006-10-25 16:04:25 +0000
@@ -80,17 +80,13 @@
 
     :return: workingtree, [relative_paths]
     """
-    def realpath(path):
-        return osutils.pathjoin(osutils.realpath(osutils.dirname(path)),
-                                osutils.basename(path))
-        
     if file_list is None or len(file_list) == 0:
         return WorkingTree.open_containing(default_branch)[0], file_list
-    tree = WorkingTree.open_containing(realpath(file_list[0]))[0]
+    tree = WorkingTree.open_containing(osutils.realpath(file_list[0]))[0]
     new_list = []
     for filename in file_list:
         try:
-            new_list.append(tree.relpath(realpath(filename)))
+            new_list.append(tree.relpath(osutils.real_parent(filename)))
         except errors.PathNotChild:
             raise errors.FileInWrongBranch(tree.branch, filename)
     return tree, new_list

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-10-23 22:43:31 +0000
+++ bzrlib/osutils.py	2006-10-25 16:04:25 +0000
@@ -1110,3 +1110,8 @@
         b += new
     return b
 
+def real_parent(path):
+    parent, base = os.path.split(path)
+    # The pathjoin for '.' is a workaround for Python bug #1213894.
+    # (initial path components aren't dereferenced)
+    return pathjoin(realpath(pathjoin('.', parent)), base)

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-10-16 01:25:46 +0000
+++ bzrlib/tests/test_osutils.py	2006-10-25 16:04:25 +0000
@@ -197,6 +197,32 @@
         self.assertFormatedDelta('1 second in the future', -1)
         self.assertFormatedDelta('2 seconds in the future', -2)
 
+    def test_real_parent(self):
+        cwd = osutils.realpath('.')
+        os.mkdir('bar')
+        bar_path = osutils.pathjoin(cwd, 'bar')
+        # Using './' to avoid bug #1213894 (first path component not
+        # dereferenced) in Python 2.4.1 and earlier
+        self.assertEqual(bar_path, osutils.realpath('./bar'))
+        os.symlink('bar', 'foo')
+        self.assertEqual(bar_path, osutils.realpath('./foo'))
+        
+        # Does not dereference terminal symlinks
+        foo_path = osutils.pathjoin(cwd, 'foo')
+        self.assertEqual(foo_path, osutils.real_parent('./foo'))
+
+        # Dereferences parent symlinks
+        os.mkdir('bar/baz')
+        baz_path = osutils.pathjoin(bar_path, 'baz')
+        self.assertEqual(baz_path, osutils.real_parent('./foo/baz'))
+
+        # Dereferences parent symlinks that are the first path element
+        self.assertEqual(baz_path, osutils.real_parent('foo/baz'))
+
+        # Dereferences parent symlinks in absolute paths
+        foo_baz_path = osutils.pathjoin(foo_path, 'baz')
+        self.assertEqual(baz_path, osutils.real_parent(foo_baz_path))
+
 
 class TestSafeUnicode(TestCase):
 

